import traceback
from typing import Optional, Callable

from cpl_core.configuration.configuration_model_abc import ConfigurationModelABC
from cpl_core.console import Console

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum


class FeatureFlagsSettings(ConfigurationModelABC):

    def __init__(self):
        ConfigurationModelABC.__init__(self)

        self._flags = {
            # modules
            FeatureFlagsEnum.admin_module.value: False,  # 02.10.2022 #48
            FeatureFlagsEnum.auto_role_module.value: True,  # 03.10.2022 #54
            FeatureFlagsEnum.base_module.value: True,  # 02.10.2022 #48
            FeatureFlagsEnum.boot_log_module.value: True,  # 02.10.2022 #48
            FeatureFlagsEnum.core_module.value: True,  # 03.10.2022 #56
            FeatureFlagsEnum.core_extension_module.value: True,  # 03.10.2022 #56
            FeatureFlagsEnum.data_module.value: True,  # 03.10.2022 #56
            FeatureFlagsEnum.database_module.value: True,  # 02.10.2022 #48
            FeatureFlagsEnum.moderator_module.value: False,  # 02.10.2022 #48
            FeatureFlagsEnum.permission_module.value: True,  # 02.10.2022 #48
            # features
            FeatureFlagsEnum.presence.value: True,  # 03.10.2022 #56
        }

    def get_flag(self, key: FeatureFlagsEnum) -> bool:
        if key.value not in self._flags:
            return False
        return self._flags[key.value]

    def _load_flag(self, settings: dict, key: FeatureFlagsEnum):
        if key.value not in settings:
            return

        self._flags[key.value] = bool(settings[key.value])

    def from_dict(self, settings: dict):
        try:
            for flag in [f.value for f in FeatureFlagsEnum]:
                self._load_flag(settings, FeatureFlagsEnum(flag))
        except Exception as e:
            Console.error(f'[ ERROR ] [ {__name__} ]: Reading error in {type(self).__name__} settings')
            Console.error(f'[ EXCEPTION ] [ {__name__} ]: {e} -> {traceback.format_exc()}')
